@@ -4,6 +4,8 @@ use Carp ();
 use Class::Load ();
 use Scalar::Util qw/blessed/;
 use Dwarf::Validator::Constraint::Default;
+use Dwarf::Validator::HashRef;
+use Dwarf::Validator::PlackRequest;
 use Dwarf::Accessor {
 	rw => [qw/query/]
 };
@@ -29,7 +31,14 @@ sub load_constraints {
 sub new {
 	my ($class, $q) = @_;
 	Carp::croak("Usage: ${class}->new(\$q)") unless $q;
-	bless { query => $q, _error => {} }, $class;
+
+	if (ref $q eq 'HASH') {
+		$q = Dwarf::Validator::HashRef->new($q);
+	} else {
+		$q = Dwarf::Validator::PlackRequest->new($q);
+	}
+
+	return bless { query => $q, _error => {} }, $class;
 }
 
 sub check {
@@ -49,9 +58,10 @@ sub check {
 				}
 			} else {
 				($key, @values) = $self->_extract_parameters_values($rule_key);
-				for my $value (@values) {
+				for my $i (0 .. @values - 1) {
+					my $value = $values[$i];
 					local $_ = $value;
-					$self->_check_param($key, $rule_name, $args);
+					$self->_check_param($key, $rule_name, $args, $i);
 				}
 			}
 		}
@@ -63,31 +73,21 @@ sub check {
 
 sub _extract_parameters_values {
 	my ($self, $key) = @_;
-
-	my $q = $self->{query};
-	my @values;
-	if (ref $key) {
-		$key = [%$key];
-		@values = [ map { $q->param($_) } @{ $key->[1] } ];
-		$key = $key->[0];
-	} else {
-		@values = $q->parameters->get_all($key);
-		@values = undef if @values == 0;
-	}
-
-	return ($key, @values);
+	$self->query->extract_parameters_values($key);
 }
 
 sub _extract_uploads_values {
 	my ($self, $key) = @_;
-	my $q = $self->{query};
-	my @values = $q->uploads->get_all($key);
-	@values = undef if @values == 0;
-	return ($key, @values);
+	$self->query->extract_uploads_values($key);
+}
+
+sub _set_param {
+	my ($self, $key, $val, $index) = @_;
+	$self->query->set_param($key, $val, $index);
 }
 
 sub _check_param {
-	my ($self, $key, $rule_name, $args) = @_;
+	my ($self, $key, $rule_name, $args, $index) = @_;
 
 	#warn "$key: ", $rule_name;
 
@@ -104,7 +104,7 @@ sub _check_param {
 		# FILTER が何か値を返す場合は元の値を上書きする
 		if ($rule_name eq 'FILTER') {
 			my $value = $code->(@$args);
-			$self->_set_param($key, $value) unless ref $value eq 'Dwarf::Validator::NullValue';
+			$self->_set_param($key, $value, $index) unless ref $value eq 'Dwarf::Validator::NullValue';
 			1;
 		} elsif ((not (defined $_ && length $_)) && $rule_name !~ /^(NOT_NULL|REQUIRED|NOT_BLANK)$/) {
 			1;
@@ -149,11 +149,6 @@ sub _rule_args {
 	return ref($rule) ? [ @$rule[ 1 .. scalar(@$rule)-1 ] ] : +[];
 }
 
-sub _set_param {
-	my ($self, $key, $val) = @_;
-	$self->{query}->parameters->set($key, $val);
-}
-
 sub is_error {
 	my ($self, $key) = @_;
 	$self->{_error}->{$key} ? 1 : 0;
